'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      Program: function Program(path, state) {
        path.traverse({
          ClassMethod: function ClassMethod(path) {
            if (path.node.key.name !== 'render') {
              return;
            }

            var opts = _extends({}, defaults, state.opts);
            var cls = path.findParent(function (path) {
              return path.isClassDeclaration();
            });

            if (!matchesSuperClass(t, cls.node.superClass, opts.superClasses)) {
              return;
            }

            var params = path.get('params').slice(0, members.length);
            if (!params.length) {
              return;
            }

            var body = path.get('body');
            var declarators = params.map(function (param, i) {
              return t.variableDeclarator(param.node, variableExpression(t, members[i]));
            });

            var declaration = t.variableDeclaration('const', declarators);
            body.unshiftContainer('body', declaration);

            params.forEach(function (param) {
              param.remove();
            });
          }
        });
      }
    }
  };
};

var defaults = {
  superClasses: ['Component', 'PureComponent']
};

var members = [{ name: 'props' }, { name: 'state', fallback: true }, { name: 'context' }];

var variableExpression = function variableExpression(t, member) {
  var memberExpression = t.memberExpression(t.thisExpression(), t.identifier(member.name));

  return member.fallback ? t.logicalExpression('||', memberExpression, t.objectExpression([])) : memberExpression;
};

var matchesSuperClass = function matchesSuperClass(t, superClass, validClasses) {
  var identifier = void 0;

  if (t.isIdentifier(superClass)) {
    identifier = superClass;
  } else if (t.isMemberExpression(superClass) && t.isIdentifier(superClass.object, { name: 'React' })) {
    identifier = superClass.property;
  }

  return identifier && validClasses.includes(identifier.name);
};